searchData={"items":[{"type":"module","title":"TigerBeetlex.Client","doc":"Message based API.\n\nThis module exposes a message based API to the TigerBeetle NIF client. The responses from the NIF\narrive in the form of messages. This allows to integrate the client in an existing process\narchitecture without spawning any other processes.","ref":"TigerBeetlex.Client.html"},{"type":"module","title":"Response decoding - TigerBeetlex.Client","doc":"When submitting a request through a function in this module, the response is received via message.\n\nAll the functions performing a TigerBeetle request return a ref which can be used to match the\nreceived response message.\n\nThe response message can be received with this receive pattern:\n\n    {:ok, request_ref} = Client.create_accounts(client, accounts)\n\n    {:ok, results} =\n      receive do\n        {:tigerbeetlex_response, ^request_ref, response} ->\n          TigerBeetlex.Response.decode(response)\n      end\n    end\n\nWhere `request_ref` is the `ref` returned when the request was submitted. `response` can then be\ndecoded using `TigerBeetlex.Response.decode/1`.\n\nThe value returned from `TigerBeetlex.Response.decode(response)` will either be `{:error, reason}`\nor `{:ok, results}`.\n\nThe type of `results` will depend on the operation that was submitted.\n\nIf the caller is a `GenServer` or similar process, it should pattern match for the response tuple\nin its `handle_info` callback.\n\n### `receive_and_decode/1`\n\nIf the response is the only message you need to receive, you can call `Client.receive_and_decode/1`\nas a shorthand for the `receive` block above.\n\n    {:ok, request_ref} = Client.create_accounts(client, accounts)\n    {:ok, results} = Client.receive_and_decode(request_ref)","ref":"TigerBeetlex.Client.html#module-response-decoding"},{"type":"function","title":"TigerBeetlex.Client.create_accounts/2","doc":"Creates a batch of accounts.\n\n`client` is a `TigerBeetlex.Client` struct.\n\n`accounts` is a list of `TigerBeetlex.Account` structs.\n\nThe decoded `results` are a list of `TigerBeetlex.CreateAccountsResult` structs\nwhich contain the index of the account list and the reason of the failure. An account has a\ncorresponding `TigerBeetlex.CreateAccountsResult` only if it fails to be created, otherwise\nthe account has been created succesfully (so a successful request returns an empty list).\n\nSee [`create_accounts`](https://docs.tigerbeetle.com/reference/requests/create_accounts/).","ref":"TigerBeetlex.Client.html#create_accounts/2"},{"type":"function","title":"Examples - TigerBeetlex.Client.create_accounts/2","doc":"alias TigerBeetlex.Account\n    alias TigerBeetlex.ID\n\n    # Successful request\n    accounts = [%Account{id: ID.generate(), ledger: 3, code: 4}]\n\n    {:ok, ref} = Client.create_accounts(client, accounts)\n\n    Client.receive_and_decode(ref)\n\n    #=> {:ok, []}\n\n    # Creation error\n    accounts = [%Account{id: ID.from_int(0), ledger: 3, code: 4}]\n\n    {:ok, ref} = Client.create_accounts(client, accounts)\n\n    Client.receive_and_decode(ref)\n\n    #=> {:ok, [%TigerBeetlex.CreateAccountsResult{index: 0, result: :id_must_not_be_zero}]}","ref":"TigerBeetlex.Client.html#create_accounts/2-examples"},{"type":"function","title":"TigerBeetlex.Client.create_transfers/2","doc":"Creates a batch of transfers.\n\n`client` is a `TigerBeetlex.Client` struct.\n\n`transfers` is a list of `TigerBeetlex.Transfer` structs.\n\nThe decoded `results` are a list of `TigerBeetlex.CreateTransfersResult` structs\nwhich contain the index of the transfer list and the reason of the failure. A transfer has a\ncorresponding `TigerBeetlex.CreateTransfersResult` only if it fails to be created, otherwise\nthe transfer has been created succesfully (so a successful request returns an empty list).\n\nSee [`create_transfers`](https://docs.tigerbeetle.com/reference/requests/create_transfers/).","ref":"TigerBeetlex.Client.html#create_transfers/2"},{"type":"function","title":"Examples - TigerBeetlex.Client.create_transfers/2","doc":"alias TigerBeetlex.ID\n    alias TigerBeetlex.Transfer\n\n    # Successful request\n    transfers = [\n      %Transfer{\n        id: ID.generate(),\n        debit_account_id: ID.from_int(42),\n        credit_account_id: ID.from_int(43),\n        ledger: 3,\n        code: 4\n        amount: 100\n      }\n    ]\n\n    {:ok, ref} = Client.create_transfers(client, transfers)\n\n    Client.receive_and_decode(ref)\n\n    #=> {:ok, []}\n\n    # Creation error\n    transfers = [\n      %Transfer{\n        id: ID.from_int(0),\n        debit_account_id: ID.from_int(42),\n        credit_account_id: ID.from_int(43),\n        ledger: 3,\n        code: 4\n        amount: 100\n      }\n    ]\n\n    {:ok, ref} = Client.create_transfers(client, transfers)\n\n    Client.receive_and_decode(ref)\n\n    #=> {:ok, [%TigerBeetlex.CreateTransfersResult{index: 0, result: :id_must_not_be_zero}]}","ref":"TigerBeetlex.Client.html#create_transfers/2-examples"},{"type":"function","title":"TigerBeetlex.Client.get_account_balances/2","doc":"Fetch a list of historical `TigerBeetlex.AccountBalance` for a given `TigerBeetlex.Account`.\n\nOnly accounts created with the `history` flag set retain historical balances. This is off by default.\n\n`client` is a `TigerBeetlex.Client` struct.\n\n`account_filter` is a `TigerBeetlex.AccountFilter` struct. The `limit` field must be set.\n\nThe decoded `results` are a list of `TigerBeetlex.AccountBalance` structs that match `account_filter`.\n\nSee [`get_account_balances`](https://docs.tigerbeetle.com/reference/requests/get_account_balances/).","ref":"TigerBeetlex.Client.html#get_account_balances/2"},{"type":"function","title":"Examples - TigerBeetlex.Client.get_account_balances/2","doc":"alias TigerBeetlex.AccountFilter\n    alias TigerBeetlex.ID\n\n    account_filter = %AccountFilter{id: ID.from_int(42), limit: 10}\n\n    {:ok, ref} = Client.get_account_balances(client, account_filter)\n\n    Client.receive_and_decode(ref)\n\n    #=> {:ok, [%TigerBeetlex.AccountBalance{}]}","ref":"TigerBeetlex.Client.html#get_account_balances/2-examples"},{"type":"function","title":"TigerBeetlex.Client.get_account_transfers/2","doc":"Fetch a list of `%TigerBeetlex.Transfer{}` involving a `%TigerBeetlex.Account{}`.\n\n`client` is a `TigerBeetlex.Client` struct.\n\n`account_filter` is a `TigerBeetlex.AccountFilter` struct. The `limit` field must be set.\n\nThe decoded `results` are a list of `TigerBeetlex.Transfer` structs that match `account_filter`.\n\nSee [`get_account_transfers`](https://docs.tigerbeetle.com/reference/requests/get_account_transfers/).","ref":"TigerBeetlex.Client.html#get_account_transfers/2"},{"type":"function","title":"Examples - TigerBeetlex.Client.get_account_transfers/2","doc":"alias TigerBeetlex.AccountFilter\n    alias TigerBeetlex.ID\n\n    account_filter = %AccountFilter{id: ID.from_int(42), limit: 10}\n\n    {:ok, ref} = Client.get_account_balances(client, account_filter)\n\n    Client.receive_and_decode(ref)\n\n    #=> {:ok, [%TigerBeetlex.Transfer{}]}","ref":"TigerBeetlex.Client.html#get_account_transfers/2-examples"},{"type":"function","title":"TigerBeetlex.Client.lookup_accounts/2","doc":"Lookup a batch of accounts.\n\n`client` is a `TigerBeetlex.Client` struct.\n\n`ids` is a list of 128-bit binaries.\n\nThe decoded `results` are a list of `TigerBeetlex.Account` structs.\n\nSee [`lookup_accounts`](https://docs.tigerbeetle.com/reference/requests/lookup_accounts/).","ref":"TigerBeetlex.Client.html#lookup_accounts/2"},{"type":"function","title":"Examples - TigerBeetlex.Client.lookup_accounts/2","doc":"alias TigerBeetlex.ID\n\n    ids = [ID.from_int(42)]\n\n    {:ok, ref} = Client.lookup_accounts(client, ids)\n\n    Client.receive_and_decode(ref)\n\n    #=> {:ok, [%TigerBeetlex.Account{}]}","ref":"TigerBeetlex.Client.html#lookup_accounts/2-examples"},{"type":"function","title":"TigerBeetlex.Client.lookup_transfers/2","doc":"Lookup a batch of transfers.\n\n`client` is a `TigerBeetlex.Client` struct.\n\n`ids` is a list of 128-bit binaries.\n\nThe decoded `results` are a list of `TigerBeetlex.Transfer` structs.\n\nSee [`lookup_transfers`](https://docs.tigerbeetle.com/reference/requests/lookup_transfers/).","ref":"TigerBeetlex.Client.html#lookup_transfers/2"},{"type":"function","title":"Examples - TigerBeetlex.Client.lookup_transfers/2","doc":"alias TigerBeetlex.ID\n\n    ids = [ID.from_int(42)]\n\n    {:ok, ref} = Client.lookup_transfers(client, ids)\n\n    Client.receive_and_decode(ref)\n\n    #=> {:ok, [%TigerBeetlex.Transfer{}]}","ref":"TigerBeetlex.Client.html#lookup_transfers/2-examples"},{"type":"function","title":"TigerBeetlex.Client.new/2","doc":"Creates a message based TigerBeetlex client.\n\nThe returned client can be safely shared between multiple processes. Each process will receive\nthe responses to the requests it submits.","ref":"TigerBeetlex.Client.html#new/2"},{"type":"function","title":"Arguments - TigerBeetlex.Client.new/2","doc":"- `cluster_id` (128-bit binary ID): - The TigerBeetle cluster id.\n- `addresses` (list of `String.t()`) - The list of node addresses. These can either be a single\ndigit (e.g. `\"3000\"`), which is interpreted as a port on `127.0.0.1`, an IP address + port (e.g.\n`\"127.0.0.1:3000\"`), or just an IP address (e.g. `\"127.0.0.1\"`), which defaults to port `3001`.","ref":"TigerBeetlex.Client.html#new/2-arguments"},{"type":"function","title":"Examples - TigerBeetlex.Client.new/2","doc":"alias TigerBeetlex.ID\n\n    {:ok, client} = Client.new(ID.from_int(0), [\"3000\"])","ref":"TigerBeetlex.Client.html#new/2-examples"},{"type":"function","title":"TigerBeetlex.Client.query_accounts/2","doc":"Query accounts by the intersection of some fields and by timestamp range.\n\n`client` is a `TigerBeetlex.Client` struct.\n\n`query_filter` is a `TigerBeetlex.QueryFilter` struct. The `limit` field must be set.\n\nThe decoded `results` are a list of `TigerBeetlex.Account` structs that match `query_filter`.\n\nSee [`query_accounts`](https://docs.tigerbeetle.com/reference/requests/query_accounts/).","ref":"TigerBeetlex.Client.html#query_accounts/2"},{"type":"function","title":"Examples - TigerBeetlex.Client.query_accounts/2","doc":"alias TigerBeetlex.QueryFilter\n\n    query_filter = %QueryFilter{ledger: 42, limit: 10}\n\n    {:ok, ref} = Client.query_accounts(client, query_filter)\n\n    Client.receive_and_decode(ref)\n\n    #=> {:ok, [%TigerBeetlex.Account{}]}","ref":"TigerBeetlex.Client.html#query_accounts/2-examples"},{"type":"function","title":"TigerBeetlex.Client.query_transfers/2","doc":"Query transfers by the intersection of some fields and by timestamp range.\n\n`client` is a `TigerBeetlex.Client` struct.\n\n`query_filter` is a `TigerBeetlex.QueryFilter` struct. The `limit` field must be set.\n\nThe decoded `results` are a list of `TigerBeetlex.Transfer` structs that match `query_filter`.\n\nSee [`query_transfers`](https://docs.tigerbeetle.com/reference/requests/query_transfers/).","ref":"TigerBeetlex.Client.html#query_transfers/2"},{"type":"function","title":"Examples - TigerBeetlex.Client.query_transfers/2","doc":"alias TigerBeetlex.QueryFilter\n\n    query_filter = %QueryFilter{ledger: 42, limit: 10}\n\n    {:ok, ref} = Client.query_transfers(client, ids)\n\n    Client.receive_and_decode(ref)\n\n    #=> {:ok, [%TigerBeetlex.Transfer{}]}","ref":"TigerBeetlex.Client.html#query_transfers/2-examples"},{"type":"function","title":"TigerBeetlex.Client.receive_and_decode/1","doc":"Utility function to receive a message response and decode it.\n\nThis is useful to emulate blocking behavior if TigerBeetlex is the only process that\ncan send messages to your process.\n\nNote that the function doesn't have a timeout and could block forever. This is expected\nsince the TigerBeetle client\n[never times out](https://docs.tigerbeetle.com/reference/sessions/#retries).\n\nSee all the other functions in this module for example usage.","ref":"TigerBeetlex.Client.html#receive_and_decode/1"},{"type":"type","title":"TigerBeetlex.Client.t/0","doc":"","ref":"TigerBeetlex.Client.html#t:t/0"},{"type":"module","title":"TigerBeetlex.Response","doc":"NIF Response parsing.\nThis module is responsible for converting a response received from the TigerBeetle NIF to either\nan error or a list of results.","ref":"TigerBeetlex.Response.html"},{"type":"function","title":"TigerBeetlex.Response.decode/1","doc":"Converts a NIF message response to a list of results.\n\nIf the response contains an error, `{:error, reason}` is returned.\n\nIf successful, it returns `{:ok, list}`. The type of the items of the list depend on the\noperation.\n\n- `create_accounts`: a list of `%TigerBeetlex.CreateAccountsResult{}`\n- `create_transfers`: a list of `%TigerBeetlex.CreateTransfersResult{}`\n- `lookup_accounts`: a list of `%TigerBeetlex.Account{}`\n- `lookup_transfers`: a list of `%TigerBeetlex.Transfer{}`\n- `get_account_transfers`: a list of `%TigerBeetlex.Transfer{}`\n- `get_account_balances`: a list of `%TigerBeetlex.AccountBalance{}`\n- `query_accounts`: a list of `%TigerBeetlex.Account{}`\n- `query_transfers`: a list of `%TigerBeetlex.Transfer{}`","ref":"TigerBeetlex.Response.html#decode/1"},{"type":"module","title":"TigerBeetlex.Connection","doc":"Blocking API.\n\nThis module exposes a blocking API to the TigerBeetle NIF client. This is obtained by spawning\nreceiver processes under a `PartitionSupervisor`. The receiver processes handle receiving messages\nfrom the message based `TigerBeetlex.Client` client and translate them in a blocking API.\n\nIf you already have a wrapping process, you can use `TigerBeetlex.Client` directly instead.","ref":"TigerBeetlex.Connection.html"},{"type":"function","title":"TigerBeetlex.Connection.create_accounts/2","doc":"Creates a batch of accounts.\n\n`name` is the same atom that was passed in the `:name` option in `start_link/1`.\n\n`accounts` is a list of `TigerBeetlex.Account` structs.\n\nIf successful, the function returns `{:ok, results}` where `results` is a list of\n`TigerBeetlex.CreateAccountsResult` structs which contain the index\nof the account list and the reason of the failure. An account has a corresponding\n`TigerBeetlex.CreateAccountsResult` only if it fails to be created, otherwise the account\nhas been created succesfully (so a successful request returns an empty list).\n\nSee [`create_accounts`](https://docs.tigerbeetle.com/reference/requests/create_accounts/).","ref":"TigerBeetlex.Connection.html#create_accounts/2"},{"type":"function","title":"Examples - TigerBeetlex.Connection.create_accounts/2","doc":"alias TigerBeetlex.Account\n    alias TigerBeetlex.ID\n\n    # Successful request\n    accounts = [%Account{id: ID.generate(), ledger: 3, code: 4}]\n\n    TigerBeetlex.Connection.create_accounts(:tb, accounts)\n    #=> {:ok, []}\n\n    # Creation error\n    accounts = [%Account{id: ID.from_int(0), ledger: 3, code: 4}]\n\n    TigerBeetlex.Connection.create_accounts(:tb, accounts)\n\n    #=> {:ok, [%TigerBeetlex.CreateAccountsResult{index: 0, result: :id_must_not_be_zero}]}","ref":"TigerBeetlex.Connection.html#create_accounts/2-examples"},{"type":"function","title":"TigerBeetlex.Connection.create_transfers/2","doc":"Creates a batch of transfers.\n\n`name` is the same atom that was passed in the `:name` option in `start_link/1`.\n\n`transfers` is a list of `TigerBeetlex.Transfer` structs.\n\nIf successful, the function returns `{:ok, results}` where `results` is a list of\n`TigerBeetlex.CreateTransfersResult` structs which contain the index\nof the transfer list and the reason of the failure. A transfer has a corresponding\n`TigerBeetlex.CreateTransfersResult` only if it fails to be created, otherwise the transfer\nhas been created succesfully (so a successful request returns an empty list).\n\nSee [`create_transfers`](https://docs.tigerbeetle.com/reference/requests/create_transfers/).","ref":"TigerBeetlex.Connection.html#create_transfers/2"},{"type":"function","title":"Examples - TigerBeetlex.Connection.create_transfers/2","doc":"alias TigerBeetlex.ID\n    alias TigerBeetlex.Transfer\n\n    # Successful request\n    transfers = [\n      %Transfer{\n        id: ID.generate(),\n        debit_account_id: ID.from_int(42),\n        credit_account_id: ID.from_int(43),\n        ledger: 3,\n        code: 4\n        amount: 100\n      }\n    ]\n\n    TigerBeetlex.Connection.create_transfers(:tb, transfers)\n    #=> {:ok, []}\n\n    # Creation error\n    transfers = [\n      %Transfer{\n        id: ID.from_int(0),\n        debit_account_id: ID.from_int(42),\n        credit_account_id: ID.from_int(43),\n        ledger: 3,\n        code: 4\n        amount: 100\n      }\n    ]\n\n    TigerBeetlex.Connection.create_transfers(:tb, transfers)\n    #=> {:ok, [%TigerBeetlex.CreateTransferError{index: 0, result: :id_must_not_be_zero}]}","ref":"TigerBeetlex.Connection.html#create_transfers/2-examples"},{"type":"function","title":"TigerBeetlex.Connection.get_account_balances/2","doc":"Fetch a list of historical `TigerBeetlex.AccountBalance` for a given `TigerBeetlex.Account`.\n\nOnly accounts created with the `history` flag set retain historical balances. This is off by default.\n\n`name` is the same atom that was passed in the `:name` option in `start_link/1`.\n\n`account_filter` is a `TigerBeetlex.AccountFilter` struct. The `limit` field must be set.\n\nIf successful, the function returns `{:ok, results}` where `results` is a list of\n`TigerBeetlex.AccountBalance` structs.\n\nSee [`get_account_balances`](https://docs.tigerbeetle.com/reference/requests/get_account_balances/).","ref":"TigerBeetlex.Connection.html#get_account_balances/2"},{"type":"function","title":"Examples - TigerBeetlex.Connection.get_account_balances/2","doc":"alias TigerBeetlex.AccountFilter\n    alias TigerBeetlex.ID\n\n    account_filter = %AccountFilter{id: ID.from_int(42), limit: 10}\n\n    TigerBeetlex.Connection.get_account_balances(:tb, account_filter)\n    #=> {:ok, [%TigerBeetlex.AccountBalance{}]}","ref":"TigerBeetlex.Connection.html#get_account_balances/2-examples"},{"type":"function","title":"TigerBeetlex.Connection.get_account_transfers/2","doc":"Fetch a list of `TigerBeetlex.Transfer` involving a `TigerBeetlex.Account`.\n\n`name` is the same atom that was passed in the `:name` option in `start_link/1`.\n\n`account_filter` is a `TigerBeetlex.AccountFilter` struct. The `limit` field must be set.\n\nIf successful, the function returns `{:ok, results}` where `results` is a list of\n`TigerBeetlex.Transfer` structs.\n\nSee [`get_account_transfers`](https://docs.tigerbeetle.com/reference/requests/get_account_transfers/).","ref":"TigerBeetlex.Connection.html#get_account_transfers/2"},{"type":"function","title":"Examples - TigerBeetlex.Connection.get_account_transfers/2","doc":"alias TigerBeetlex.AccountFilter\n    alias TigerBeetlex.ID\n\n    account_filter = %AccountFilter{id: ID.from_int(42), limit: 10}\n\n    TigerBeetlex.Connection.get_account_transfers(:tb, account_filter)\n    #=> {:ok, [%TigerBeetlex.Transfer{}]}","ref":"TigerBeetlex.Connection.html#get_account_transfers/2-examples"},{"type":"function","title":"TigerBeetlex.Connection.lookup_accounts/2","doc":"Lookup a batch of accounts.\n\n`name` is the same atom that was passed in the `:name` option in `start_link/1`.\n\n`ids` is a list of 128-bit binaries.\n\nIf successful, the function returns `{:ok, results}` where `results` is a list of\n`TigerBeetlex.Account` structs. If an id in the list does not correspond to an existing\naccount, it will simply be skipped, so the result can have less accounts then the provided\nids.\n\nSee [`lookup_accounts`](https://docs.tigerbeetle.com/reference/requests/lookup_accounts/).","ref":"TigerBeetlex.Connection.html#lookup_accounts/2"},{"type":"function","title":"Examples - TigerBeetlex.Connection.lookup_accounts/2","doc":"alias TigerBeetlex.ID\n\n    ids = [ID.from_int(42)]\n\n    TigerBeetlex.Connection.lookup_accounts(:tb, ids)\n    #=> {:ok, [%TigerBeetlex.Account{}]}","ref":"TigerBeetlex.Connection.html#lookup_accounts/2-examples"},{"type":"function","title":"TigerBeetlex.Connection.lookup_transfers/2","doc":"Lookup a batch of transfers.\n\n`name` is the same atom that was passed in the `:name` option in `start_link/1`.\n\n`ids` is a list of 128-bit binaries.\n\nIf successful, the function returns `{:ok, results}` where `results` is a list of\n`TigerBeetlex.Transfer` structs. If an id in the list does not correspond to an existing\ntransfer, it will simply be skipped, so the result could have less transfers then the provided\nids.\n\nSee [`lookup_transfers`](https://docs.tigerbeetle.com/reference/requests/lookup_transfers/).","ref":"TigerBeetlex.Connection.html#lookup_transfers/2"},{"type":"function","title":"Examples - TigerBeetlex.Connection.lookup_transfers/2","doc":"alias TigerBeetlex.ID\n\n    ids = [ID.from_int(43)]\n\n    TigerBeetlex.Connection.lookup_transfers(:tb, ids)\n    #=> {:ok, [%TigerBeetlex.Transfer{}]}","ref":"TigerBeetlex.Connection.html#lookup_transfers/2-examples"},{"type":"function","title":"TigerBeetlex.Connection.query_accounts/2","doc":"Query accounts by the intersection of some fields and by timestamp range.\n\n`name` is the same atom that was passed in the `:name` option in `start_link/1`.\n\n`query_filter` is a `TigerBeetlex.QueryFilter` struct. The `limit` field must be set.\n\nIf successful, the function returns `{:ok, results}` where `results` is a list of\n`TigerBeetlex.Account` structs.\n\nSee [`query_accounts`](https://docs.tigerbeetle.com/reference/requests/query_accounts/).","ref":"TigerBeetlex.Connection.html#query_accounts/2"},{"type":"function","title":"Examples - TigerBeetlex.Connection.query_accounts/2","doc":"alias TigerBeetlex.QueryFilter\n\n    query_filter = %QueryFilter{ledger: 42, limit: 10}\n\n    TigerBeetlex.Connection.query_accounts(:tb, query_filter)\n    #=> {:ok, [%TigerBeetlex.Account{}]}","ref":"TigerBeetlex.Connection.html#query_accounts/2-examples"},{"type":"function","title":"TigerBeetlex.Connection.query_transfers/2","doc":"Query transfers by the intersection of some fields and by timestamp range.\n\n`name` is the same atom that was passed in the `:name` option in `start_link/1`.\n\n`query_filter` is a `TigerBeetlex.QueryFilter` struct. The `limit` field must be set.\n\nIf successful, the function returns `{:ok, results}` where `results` is a list of\n`TigerBeetlex.Transfer` structs.\n\nSee [`query_transfers`](https://docs.tigerbeetle.com/reference/requests/query_transfers/).","ref":"TigerBeetlex.Connection.html#query_transfers/2"},{"type":"function","title":"Examples - TigerBeetlex.Connection.query_transfers/2","doc":"alias TigerBeetlex.QueryFilter\n\n    query_filter = %QueryFilter{ledger: 42, limit: 10}\n\n    TigerBeetlex.Connection.query_transfers(:tb, query_filter)\n    #=> {:ok, [%TigerBeetlex.Transfer{}]}","ref":"TigerBeetlex.Connection.html#query_transfers/2-examples"},{"type":"function","title":"TigerBeetlex.Connection.start_link/1","doc":"Starts a managed TigerBeetlex connection.\n\nThis creates N receiver processes (where N can be controlled by\npassing options to the underlying `PartitionSupervisor`) under a `PartitionSupervisor`.\n\nOn success, the it returns the pid of the `PartitionSupervisor`. Note that this is not\nwhat you have to pass to all the other functions in this module as first argument,\nrather the `name` atom passed as option to `start_link/1` has to be passed.","ref":"TigerBeetlex.Connection.html#start_link/1"},{"type":"function","title":"Options - TigerBeetlex.Connection.start_link/1","doc":"These are the TigerBeetlex-specific options supported by this function:\n\n* `:name` (`t:atom/0`) - Required. The name of the Connection process. The same atom has to be passed to all the other functions\n  as first argument.\n\n* `:cluster_id` (`t:TigerBeetlex.Types.id_128/0`) - Required. The TigerBeetle cluster id.\n\n* `:addresses` (list of `t:String.t/0`) - Required. The list of node addresses. These can either be a single digit (e.g. `\"3000\"`), which is\n  interpreted as a port on `127.0.0.1`, an IP address + port (e.g. `\"127.0.0.1:3000\"`), or just\n  an IP address (e.g. `\"127.0.0.1\"`), which defaults to port `3001`.\n\n\n\nThe rest of the options are forwarded to `PartitionSupervisor.start_link/1`. For example, to\nstart a pool of five receivers, you can use the `:partitions` option:\n\n    TigerBeetlex.Connection.start_link(\n      partitions: 5,\n      # ...\n    )","ref":"TigerBeetlex.Connection.html#start_link/1-options"},{"type":"function","title":"Examples - TigerBeetlex.Connection.start_link/1","doc":"alias TigerBeetlex.Connection\n    alias TigerBeetlex.ID\n\n    # Start the TigerBeetlex connection\n    {:ok, pid} =\n      Connection.start_link(\n        cluster_id: ID.from_int(0),\n        addresses: [\"3000\"],\n        name: :tb\n      )","ref":"TigerBeetlex.Connection.html#start_link/1-examples"},{"type":"module","title":"TigerBeetlex.Account","doc":"See [Account](https://docs.tigerbeetle.com/reference/account#).","ref":"TigerBeetlex.Account.html"},{"type":"function","title":"TigerBeetlex.Account.from_binary/1","doc":"Creates a `TigerBeetlex.Account` struct from its binary representation.","ref":"TigerBeetlex.Account.html#from_binary/1"},{"type":"type","title":"TigerBeetlex.Account.t/0","doc":"","ref":"TigerBeetlex.Account.html#t:t/0"},{"type":"function","title":"TigerBeetlex.Account.to_binary/1","doc":"Converts a `TigerBeetlex.Account` struct to its binary representation.","ref":"TigerBeetlex.Account.html#to_binary/1"},{"type":"module","title":"TigerBeetlex.AccountBalance","doc":"See [AccountBalance](https://docs.tigerbeetle.com/reference/account-balance#).","ref":"TigerBeetlex.AccountBalance.html"},{"type":"function","title":"TigerBeetlex.AccountBalance.from_binary/1","doc":"Creates a `TigerBeetlex.AccountBalance` struct from its binary representation.","ref":"TigerBeetlex.AccountBalance.html#from_binary/1"},{"type":"type","title":"TigerBeetlex.AccountBalance.t/0","doc":"","ref":"TigerBeetlex.AccountBalance.html#t:t/0"},{"type":"function","title":"TigerBeetlex.AccountBalance.to_binary/1","doc":"Converts a `TigerBeetlex.AccountBalance` struct to its binary representation.","ref":"TigerBeetlex.AccountBalance.html#to_binary/1"},{"type":"module","title":"TigerBeetlex.AccountFilter","doc":"See [AccountFilter](https://docs.tigerbeetle.com/reference/account-filter#).","ref":"TigerBeetlex.AccountFilter.html"},{"type":"function","title":"TigerBeetlex.AccountFilter.from_binary/1","doc":"Creates a `TigerBeetlex.AccountFilter` struct from its binary representation.","ref":"TigerBeetlex.AccountFilter.html#from_binary/1"},{"type":"type","title":"TigerBeetlex.AccountFilter.t/0","doc":"","ref":"TigerBeetlex.AccountFilter.html#t:t/0"},{"type":"function","title":"TigerBeetlex.AccountFilter.to_binary/1","doc":"Converts a `TigerBeetlex.AccountFilter` struct to its binary representation.","ref":"TigerBeetlex.AccountFilter.html#to_binary/1"},{"type":"module","title":"TigerBeetlex.AccountFilterFlags","doc":"See [AccountFilterFlags](https://docs.tigerbeetle.com/reference/account-filter#flags).","ref":"TigerBeetlex.AccountFilterFlags.html"},{"type":"function","title":"TigerBeetlex.AccountFilterFlags.from_binary/1","doc":"Creates a `TigerBeetlex.AccountFilterFlags` struct from its binary representation.","ref":"TigerBeetlex.AccountFilterFlags.html#from_binary/1"},{"type":"type","title":"TigerBeetlex.AccountFilterFlags.t/0","doc":"","ref":"TigerBeetlex.AccountFilterFlags.html#t:t/0"},{"type":"function","title":"TigerBeetlex.AccountFilterFlags.to_binary/1","doc":"Converts a `TigerBeetlex.AccountFilterFlags` struct to its binary represenation.","ref":"TigerBeetlex.AccountFilterFlags.html#to_binary/1"},{"type":"module","title":"TigerBeetlex.AccountFlags","doc":"See [AccountFlags](https://docs.tigerbeetle.com/reference/account#flags).","ref":"TigerBeetlex.AccountFlags.html"},{"type":"function","title":"TigerBeetlex.AccountFlags.from_binary/1","doc":"Creates a `TigerBeetlex.AccountFlags` struct from its binary representation.","ref":"TigerBeetlex.AccountFlags.html#from_binary/1"},{"type":"type","title":"TigerBeetlex.AccountFlags.t/0","doc":"","ref":"TigerBeetlex.AccountFlags.html#t:t/0"},{"type":"function","title":"TigerBeetlex.AccountFlags.to_binary/1","doc":"Converts a `TigerBeetlex.AccountFlags` struct to its binary represenation.","ref":"TigerBeetlex.AccountFlags.html#to_binary/1"},{"type":"module","title":"TigerBeetlex.CreateAccountsResult","doc":"See [CreateAccountsResult](https://docs.tigerbeetle.com/reference/requests/create_accounts#).","ref":"TigerBeetlex.CreateAccountsResult.html"},{"type":"function","title":"TigerBeetlex.CreateAccountsResult.from_binary/1","doc":"Creates a `TigerBeetlex.CreateAccountsResult` struct from its binary representation.","ref":"TigerBeetlex.CreateAccountsResult.html#from_binary/1"},{"type":"type","title":"TigerBeetlex.CreateAccountsResult.t/0","doc":"","ref":"TigerBeetlex.CreateAccountsResult.html#t:t/0"},{"type":"function","title":"TigerBeetlex.CreateAccountsResult.to_binary/1","doc":"Converts a `TigerBeetlex.CreateAccountsResult` struct to its binary representation.","ref":"TigerBeetlex.CreateAccountsResult.html#to_binary/1"},{"type":"module","title":"TigerBeetlex.CreateTransfersResult","doc":"See [CreateTransfersResult](https://docs.tigerbeetle.com/reference/requests/create_transfers#).","ref":"TigerBeetlex.CreateTransfersResult.html"},{"type":"function","title":"TigerBeetlex.CreateTransfersResult.from_binary/1","doc":"Creates a `TigerBeetlex.CreateTransfersResult` struct from its binary representation.","ref":"TigerBeetlex.CreateTransfersResult.html#from_binary/1"},{"type":"type","title":"TigerBeetlex.CreateTransfersResult.t/0","doc":"","ref":"TigerBeetlex.CreateTransfersResult.html#t:t/0"},{"type":"function","title":"TigerBeetlex.CreateTransfersResult.to_binary/1","doc":"Converts a `TigerBeetlex.CreateTransfersResult` struct to its binary representation.","ref":"TigerBeetlex.CreateTransfersResult.html#to_binary/1"},{"type":"module","title":"TigerBeetlex.QueryFilter","doc":"See [QueryFilter](https://docs.tigerbeetle.com/reference/query-filter#).","ref":"TigerBeetlex.QueryFilter.html"},{"type":"function","title":"TigerBeetlex.QueryFilter.from_binary/1","doc":"Creates a `TigerBeetlex.QueryFilter` struct from its binary representation.","ref":"TigerBeetlex.QueryFilter.html#from_binary/1"},{"type":"type","title":"TigerBeetlex.QueryFilter.t/0","doc":"","ref":"TigerBeetlex.QueryFilter.html#t:t/0"},{"type":"function","title":"TigerBeetlex.QueryFilter.to_binary/1","doc":"Converts a `TigerBeetlex.QueryFilter` struct to its binary representation.","ref":"TigerBeetlex.QueryFilter.html#to_binary/1"},{"type":"module","title":"TigerBeetlex.QueryFilterFlags","doc":"See [QueryFilterFlags](https://docs.tigerbeetle.com/reference/query-filter#flags).","ref":"TigerBeetlex.QueryFilterFlags.html"},{"type":"function","title":"TigerBeetlex.QueryFilterFlags.from_binary/1","doc":"Creates a `TigerBeetlex.QueryFilterFlags` struct from its binary representation.","ref":"TigerBeetlex.QueryFilterFlags.html#from_binary/1"},{"type":"type","title":"TigerBeetlex.QueryFilterFlags.t/0","doc":"","ref":"TigerBeetlex.QueryFilterFlags.html#t:t/0"},{"type":"function","title":"TigerBeetlex.QueryFilterFlags.to_binary/1","doc":"Converts a `TigerBeetlex.QueryFilterFlags` struct to its binary represenation.","ref":"TigerBeetlex.QueryFilterFlags.html#to_binary/1"},{"type":"module","title":"TigerBeetlex.Transfer","doc":"See [Transfer](https://docs.tigerbeetle.com/reference/transfer#).","ref":"TigerBeetlex.Transfer.html"},{"type":"function","title":"TigerBeetlex.Transfer.amount_max/0","doc":"The maximum amount for a transfer. Used in Two-Phase transfers to post the full pending amount.","ref":"TigerBeetlex.Transfer.html#amount_max/0"},{"type":"function","title":"TigerBeetlex.Transfer.from_binary/1","doc":"Creates a `TigerBeetlex.Transfer` struct from its binary representation.","ref":"TigerBeetlex.Transfer.html#from_binary/1"},{"type":"type","title":"TigerBeetlex.Transfer.t/0","doc":"","ref":"TigerBeetlex.Transfer.html#t:t/0"},{"type":"function","title":"TigerBeetlex.Transfer.to_binary/1","doc":"Converts a `TigerBeetlex.Transfer` struct to its binary representation.","ref":"TigerBeetlex.Transfer.html#to_binary/1"},{"type":"module","title":"TigerBeetlex.TransferFlags","doc":"See [TransferFlags](https://docs.tigerbeetle.com/reference/transfer#flags).","ref":"TigerBeetlex.TransferFlags.html"},{"type":"function","title":"TigerBeetlex.TransferFlags.from_binary/1","doc":"Creates a `TigerBeetlex.TransferFlags` struct from its binary representation.","ref":"TigerBeetlex.TransferFlags.html#from_binary/1"},{"type":"type","title":"TigerBeetlex.TransferFlags.t/0","doc":"","ref":"TigerBeetlex.TransferFlags.html#t:t/0"},{"type":"function","title":"TigerBeetlex.TransferFlags.to_binary/1","doc":"Converts a `TigerBeetlex.TransferFlags` struct to its binary represenation.","ref":"TigerBeetlex.TransferFlags.html#to_binary/1"},{"type":"module","title":"TigerBeetlex.ID","doc":"Utility functions to generate TigerBeetle Time-Based Identifiers.","ref":"TigerBeetlex.ID.html"},{"type":"function","title":"TigerBeetlex.ID.from_int/1","doc":"Converts an integer to a 128 bit binary id.\n\nThe integer is formatted in little endian format so that the correct ordering is preserved in\nTigerBeetle LSM trees.","ref":"TigerBeetlex.ID.html#from_int/1"},{"type":"function","title":"TigerBeetlex.ID.generate/0","doc":"Generate a TigerBeetle Time-Based Identifier.\n\nThis is a lexicographically-sortable time-based monotonic identifier. See\n[TigerBeetle docs](https://docs.tigerbeetle.com/coding/data-modeling/#tigerbeetle-time-based-identifiers-recommended)\nfor the complete explanation of the format.","ref":"TigerBeetlex.ID.html#generate/0"},{"type":"module","title":"TigerBeetlex.Types","doc":"Common typespecs.\n\nThis module provides common types that can be used in any part of the TigerBeetlex library.","ref":"TigerBeetlex.Types.html"},{"type":"type","title":"TigerBeetlex.Types.client/0","doc":"","ref":"TigerBeetlex.Types.html#t:client/0"},{"type":"type","title":"TigerBeetlex.Types.id_128/0","doc":"","ref":"TigerBeetlex.Types.html#t:id_128/0"},{"type":"type","title":"TigerBeetlex.Types.init_client_error/0","doc":"","ref":"TigerBeetlex.Types.html#t:init_client_error/0"},{"type":"type","title":"TigerBeetlex.Types.partition_supervisor_start_option/0","doc":"","ref":"TigerBeetlex.Types.html#t:partition_supervisor_start_option/0"},{"type":"type","title":"TigerBeetlex.Types.request_error/0","doc":"","ref":"TigerBeetlex.Types.html#t:request_error/0"},{"type":"type","title":"TigerBeetlex.Types.start_option/0","doc":"","ref":"TigerBeetlex.Types.html#t:start_option/0"},{"type":"type","title":"TigerBeetlex.Types.start_options/0","doc":"","ref":"TigerBeetlex.Types.html#t:start_options/0"},{"type":"type","title":"TigerBeetlex.Types.tigerbeetlex_connection_start_option/0","doc":"","ref":"TigerBeetlex.Types.html#t:tigerbeetlex_connection_start_option/0"},{"type":"extras","title":"Walkthrough","doc":"# Walkthrough\n\n```elixir\nMix.install([\n  {:tigerbeetlex, \"~> 0.16.43\"}\n])\n```","ref":"walkthrough.html"},{"type":"extras","title":"Introduction - Walkthrough","doc":"This walkthrough follows the same format the [official TigerBeetle clients](https://docs.tigerbeetle.com/coding/clients/python/) use. It uses the blocking API (`TigerBeetlex.Connection`) for brevity's sake but everything applies identically to the message based API (modulo initialization and response reception).\n\nThe walkthrough can also be executed in Livebook.","ref":"walkthrough.html#introduction"},{"type":"extras","title":"Setup - Walkthrough","doc":"In your `mix.exs` add TigerBeetlex as a dependency\n\n```elixir\n{:tigerbeetlex, \"~> 0.16.43\"}\n```","ref":"walkthrough.html#setup"},{"type":"extras","title":"Creating a Client - Walkthrough","doc":"A client is created with a cluster ID and replica addresses for all replicas in the cluster. The\n cluster ID and replica addresses are both chosen by the system that starts the TigerBeetle cluster.\n\nClients are thread-safe and a single instance should be shared between multiple concurrent tasks. In the case of `TigerBeetlex.Connection`, the client is stored in the state of the underlying processes, so it's already shared across callers.\n\nMultiple clients are useful when connecting to more than one TigerBeetle cluster.\n\nIn this example the cluster ID is `0` and there is one replica. The address is read from the\n`TB_ADDRESS` environment variable and defaults to port `3000`.\n\n```elixir\nalias TigerBeetlex.Connection\n\naddress = System.get_env(\"TB_ADDRESS\", \"3000\")\n\n{:ok, _pid} = Connection.start_link(name: :tb, cluster_id: <<0::128>>, addresses: [address])\n```\n\nThe following are valid addresses:\n\n* `3000` (interpreted as `127.0.0.1:3000`)\n* `127.0.0.1:3000` (interpreted as `127.0.0.1:3000`)\n* `127.0.0.1` (interpreted as `127.0.0.1:3001`, `3001` is the default port)","ref":"walkthrough.html#creating-a-client"},{"type":"extras","title":"Creating Accounts - Walkthrough","doc":"See details for account fields in the [Accounts reference](https://docs.tigerbeetle.com/reference/account/).\n\n```elixir\nalias TigerBeetlex.Account\nalias TigerBeetlex.Connection\nalias TigerBeetlex.ID\n\naccount =\n  %Account{\n    # TigerBeetle time-based ID.\n    id: ID.generate(),\n    ledger: 1,\n    code: 718\n  }\n\n{:ok, _account_errors} = Connection.create_accounts(:tb, [account])\n# Error handling omitted.\n```\n\nSee details for the recommended ID scheme in [time-based identifiers](https://docs.tigerbeetle.com/coding/data-modeling/#tigerbeetle-time-based-identifiers-recommended).\n\n<!-- livebook:{\"break_markdown\":true} -->","ref":"walkthrough.html#creating-accounts"},{"type":"extras","title":"Account Flags - Walkthrough","doc":"The account `flags` value is a struct with binary fields. See details for these flags in the [Accounts reference](https://docs.tigerbeetle.com/reference/account#flags).\n\nTo toggle behavior for an account, set the relative field to `true`:\n\n* `AccountFlags.linked`\n* `AccountFlags.debits_must_not_exceed_credits`\n* `AccountFlags.credits_must_not_exceed_credits`\n* `AccountFlags.history`\n\nFor example, to link two accounts where the first account additionally has the `debits_must_not_exceed_credits` constraint:\n\n```elixir\nalias TigerBeetlex.AccountFlags\n\naccount0 =\n  %Account{\n    id: ID.from_int(100),\n    ledger: 1,\n    code: 1,\n    flags: %AccountFlags{linked: true, debits_must_not_exceed_credits: true}\n  }\n\naccount1 =\n  %Account{\n    id: ID.from_int(101),\n    ledger: 1,\n    code: 1,\n    flags: %AccountFlags{history: true}\n  }\n\n{:ok, _account_errors} = Connection.create_accounts(:tb, [account0, account1])\n# Error handling omitted.\n```","ref":"walkthrough.html#account-flags"},{"type":"extras","title":"Response and Errors - Walkthrough","doc":"The response is an empty list if all accounts were created successfully. If the response is non-empty, each struct in the response list contains error information for an account that failed. The error struct contains an error result as atom and the index of the account in the request batch.\n\nSee all error conditions in the [create_accounts reference](https://docs.tigerbeetle.com/reference/requests/create_accounts/).\n\n```elixir\nalias TigerBeetlex.CreateAccountsResult\n\naccount0 =\n  %Account{\n    id: ID.from_int(102),\n    ledger: 1,\n    code: 1\n  }\n\naccount1 =\n  %Account{\n    id: ID.from_int(103),\n    ledger: 1,\n    code: 1\n  }\n\naccount2 =\n  %Account{\n    id: ID.from_int(104),\n    ledger: 1,\n    code: 1\n  }\n\n{:ok, account_errors} = Connection.create_accounts(:tb, [account0, account1, account2])\n\nEnum.each(account_errors, fn\n  %CreateAccountsResult{result: :exists} = error ->\n    IO.puts(\"Batch account at index #{error.index} already exists\")\n\n  error ->\n    IO.puts(\"Batch account at index #{error.index} failed to create: #{error.result}\")\nend)\n```\n\nTo handle errors you can compare the result atom contained in the struct with the ones present in the `create_accounts` reference above.","ref":"walkthrough.html#response-and-errors"},{"type":"extras","title":"Account Lookup - Walkthrough","doc":"Account lookup is batched, like account creation. Pass in all IDs to fetch. The account for each matched ID is returned.\n\nIf no account matches an ID, no struct is returned for that account. So the order of accounts in the response is not necessarily the same as the order of IDs in the request. You can refer to the ID field in the response to distinguish accounts.\n\n```elixir\n{:ok, accounts} = Connection.lookup_accounts(:tb, [ID.from_int(100), ID.from_int(101)])\n```","ref":"walkthrough.html#account-lookup"},{"type":"extras","title":"Create Transfers - Walkthrough","doc":"This creates a journal entry between two accounts.\n\nSee details for transfer fields in the [Transfers reference](https://docs.tigerbeetle.com/reference/transfer/).\n\n```elixir\nalias TigerBeetlex.Transfer\n\ntransfers = [\n  %Transfer{\n    # TigerBeetle time-based ID.\n    id: ID.generate(),\n    debit_account_id: ID.from_int(102),\n    credit_account_id: ID.from_int(103),\n    amount: 10,\n    ledger: 1,\n    code: 720\n  }\n]\n\n{:ok, _transfer_errors} = Connection.create_transfers(:tb, transfers)\n# Error handling omitted.\n```\n\nSee details for the recommended ID scheme in [time-based identifiers](https://docs.tigerbeetle.com/coding/data-modeling/#tigerbeetle-time-based-identifiers-recommended).\n\n<!-- livebook:{\"break_markdown\":true} -->","ref":"walkthrough.html#create-transfers"},{"type":"extras","title":"Response and Errors - Walkthrough","doc":"The response is an empty list if all transfers were created successfully. If the response is non-empty, each struct in the response list contains error information for a transfer that failed. The error struct contains an error result as atom and the index of the transfer in the request batch.\n\nSee all error conditions in the [create_transfers reference](https://docs.tigerbeetle.com/reference/requests/create_transfers/).\n\n```elixir\nalias TigerBeetlex.CreateTransfersResult\n\ntransfers = [\n  %Transfer{\n    id: ID.from_int(1),\n    debit_account_id: ID.from_int(101),\n    credit_account_id: ID.from_int(102),\n    amount: 10,\n    ledger: 1,\n    code: 1\n  },\n  %Transfer{\n    id: ID.from_int(2),\n    debit_account_id: ID.from_int(101),\n    credit_account_id: ID.from_int(102),\n    amount: 10,\n    ledger: 1,\n    code: 1\n  },\n  %Transfer{\n    id: ID.from_int(3),\n    debit_account_id: ID.from_int(101),\n    credit_account_id: ID.from_int(102),\n    amount: 10,\n    ledger: 1,\n    code: 1\n  }\n]\n\n{:ok, transfer_errors} = Connection.create_transfers(:tb, transfers)\n\nEnum.each(transfer_errors, fn\n  %CreateTransfersResult{result: :exists} = error ->\n    IO.puts(\"Batch transfer at index #{error.index} already exists\")\n\n  error ->\n    IO.puts(\"Batch transfer at index #{error.index} failed to create: #{error.result}\")\nend)\n```\n\nTo handle errors you can compare the result atom contained in the struct with the ones present in the `create_transfers` reference above.","ref":"walkthrough.html#response-and-errors-1"},{"type":"extras","title":"Batching - Walkthrough","doc":"TigerBeetle performance is maximized when you batch API requests. A client instance shared across multiple processes can automatically batch concurrent requests, but the application must still send as many events as possible in a single call. For example, if you insert 1 million transfers sequentially, one at a time, the insert rate will be a _fraction_ of the potential, because the client will wait for a reply between each one.\n\n```elixir\n# List of transfer to create.\nbatch = []\n\nEnum.each(batch, fn transfer ->\n  {:ok, _transfer_errors} = Connection.create_transfers(:tb, [transfer])\n  # Error handling omitted\nend)\n```\n\nInstead, **always batch as much as you can**. The maximum batch size is set in the TigerBeetle server. The default is 8189.\n\n```elixir\n# List of transfer to create.\nbatch = []\n# FIXME\nbatch_size = 8189\n\nbatch\n|> Enum.chunk_every(batch_size)\n|> Enum.each(fn transfers ->\n  {:ok, _transfer_errors} = Connection.create_transfers(:tb, transfers)\n  # Error handling omitted\nend)\n```","ref":"walkthrough.html#batching"},{"type":"extras","title":"Queues and Workers - Walkthrough","doc":"If you are making requests to TigerBeetle from workers pulling jobs from a queue, you can batch requests to TigerBeetle by having the worker act on multiple jobs from the queue at once rather than one at a time. i.e. pulling multiple jobs from the queue rather than just one.","ref":"walkthrough.html#queues-and-workers"},{"type":"extras","title":"Transfer Flags - Walkthrough","doc":"The transfer `flags` value is a struct with boolean fields. See details for these flags in the [Accounts reference](https://docs.tigerbeetle.com/reference/transfer#flags).\n\nTo toggle behavior for an account, set the relative field to `true`:\n\n* `AccountFlags.linked`\n* `AccountFlags.pending`\n* `AccountFlags.post_pending_transfer`\n* `AccountFlags.void_pending_transfer`\n\nFor example, to link `transfer0` and `transfer1`:\n\n```elixir\nalias TigerBeetlex.TransferFlags\n\ntransfer0 =\n  %Transfer{\n    # TigerBeetle time-based ID.\n    id: ID.from_int(4),\n    debit_account_id: ID.from_int(102),\n    credit_account_id: ID.from_int(103),\n    amount: 10,\n    ledger: 1,\n    code: 720,\n    flags: %TransferFlags{linked: true}\n  }\n\ntransfer1 =\n  %Transfer{\n    # TigerBeetle time-based ID.\n    id: ID.from_int(5),\n    debit_account_id: ID.from_int(102),\n    credit_account_id: ID.from_int(103),\n    amount: 10,\n    ledger: 1,\n    code: 720\n  }\n\n{:ok, _transfer_errors} = Connection.create_transfers(:tb, [transfer0, transfer1])\n# Error handling omitted\n```","ref":"walkthrough.html#transfer-flags"},{"type":"extras","title":"Two-Phase Transfers - Walkthrough","doc":"Two-phase transfers are supported natively by toggling the appropriate flag. TigerBeetle will then adjust the `credits_pending` and `debits_pending` fields of the appropriate accounts. A corresponding post pending transfer then needs to be sent to post or void the transfer.\n\n#### Post a Pending Transfer\n\nWith `flags` set to `post_pending_transfer`, TigerBeetle will post the transfer. TigerBeetle will atomically roll back the changes to `debits_pending` and `credits_pending` of the appropriate accounts and apply them to the `debits_posted` and `credits_posted` balances.\n\n```elixir\ntransfer0 =\n  %Transfer{\n    # TigerBeetle time-based ID.\n    id: ID.from_int(6),\n    debit_account_id: ID.from_int(102),\n    credit_account_id: ID.from_int(103),\n    amount: 10,\n    ledger: 1,\n    code: 720,\n    flags: %TransferFlags{pending: true}\n  }\n\n{:ok, _transfer_errors} = Connection.create_transfers(:tb, [transfer0])\n# Error handling omitted\n\ntransfer1 =\n  %Transfer{\n    # TigerBeetle time-based ID.\n    id: ID.from_int(7),\n    debit_account_id: ID.from_int(102),\n    credit_account_id: ID.from_int(103),\n    # Post the entire pending amount.\n    amount: Transfer.amount_max(),\n    pending_id: ID.from_int(6),\n    ledger: 1,\n    code: 720,\n    flags: %TransferFlags{post_pending_transfer: true}\n  }\n\n{:ok, _transfer_errors} = Connection.create_transfers(:tb, [transfer1])\n# Error handling omitted\n```\n\n#### Void a Pending Transfer\n\nIn contrast, with flags set to `void_pending_transfer`, TigerBeetle will void the transfer. TigerBeetle will roll back the changes to `debits_pending` and `credits_pending` of the appropriate accounts and not apply them to the `debits_posted` and `credits_posted` balances.\n\n```elixir\ntransfer0 =\n  %Transfer{\n    # TigerBeetle time-based ID.\n    id: ID.from_int(8),\n    debit_account_id: ID.from_int(102),\n    credit_account_id: ID.from_int(103),\n    amount: 10,\n    ledger: 1,\n    code: 720,\n    flags: %TransferFlags{pending: true}\n  }\n\n{:ok, _transfer_errors} = Connection.create_transfers(:tb, [transfer0])\n# Error handling omitted\n\ntransfer1 =\n  %Transfer{\n    # TigerBeetle time-based ID.\n    id: ID.from_int(9),\n    debit_account_id: ID.from_int(102),\n    credit_account_id: ID.from_int(103),\n    amount: 10,\n    pending_id: ID.from_int(8),\n    ledger: 1,\n    code: 720,\n    flags: %TransferFlags{void_pending_transfer: true}\n  }\n\n{:ok, _transfer_errors} = Connection.create_transfers(:tb, [transfer1])\n# Error handling omitted\n```","ref":"walkthrough.html#two-phase-transfers"},{"type":"extras","title":"Transfer Lookup - Walkthrough","doc":"NOTE: While transfer lookup exists, it is not a flexible query API. The TigerBeetle team is developing query APIs and there will be new methods for querying transfers in the future.\n\nTransfer lookup is batched, like transfer creation. Pass in all IDs to fetch, and matched transfers are returned.\n\nIf no transfer matches an id, no struct is returned for that transfer. So the order of transfers in the response is not necessarily the same as the order of IDs in the request. You can refer to the ID field in the response to distinguish transfers.\n\n```elixir\n{:ok, transfers} = Connection.lookup_transfers(:tb, [ID.from_int(1), ID.from_int(2)])\n```","ref":"walkthrough.html#transfer-lookup"},{"type":"extras","title":"Get Account Transfers - Walkthrough","doc":"NOTE: This is a preview API that is subject to breaking changes once TigerBeetle has a stable querying API.\n\nFetches the transfers involving a given account, allowing basic filter and pagination capabilities.\n\nThe transfers in the response are sorted by `timestamp` in chronological or reverse-chronological order.\n\n```elixir\nalias TigerBeetlex.AccountFilter\nalias TigerBeetlex.AccountFilterFlags\n\nfilter =\n  %AccountFilter{\n    account_id: ID.from_int(101),\n    # No filter by UserData.\n    user_data_128: <<0::128>>,\n    user_data_64: 0,\n    user_data_32: 0,\n    # No filter by Code.\n    code: 0,\n    # No filter by Timestamp.\n    timestamp_min: 0,\n    # No filter by Timestamp.\n    timestamp_max: 0,\n    # Limit to ten transfers at most.\n    limit: 10,\n    flags: %AccountFilterFlags{\n      # Include transfer from the debit side.\n      debits: true,\n      # Include transfer from the credit side.\n      credits: true,\n      # Sort by timestamp in reverse-chronological order.\n      reversed: true\n    }\n  }\n\n{:ok, account_transfers} = Connection.get_account_transfers(:tb, filter)\n```","ref":"walkthrough.html#get-account-transfers"},{"type":"extras","title":"Get Account Balances - Walkthrough","doc":"NOTE: This is a preview API that is subject to breaking changes once TigerBeetle has a stable querying API.\n\nFetches the point-in-time balances of a given account, allowing basic filter and pagination capabilities.\n\nOnly accounts created with the flag [`history`](https://docs.tigerbeetle.com/reference/account/#flagshistory) set retain historical balances.\n\nThe balances in the response are sorted by `timestamp` in chronological or reverse-chronological order.\n\n```elixir\nfilter =\n  %AccountFilter{\n    account_id: ID.from_int(101),\n    # No filter by UserData.\n    user_data_128: <<0::128>>,\n    user_data_64: 0,\n    user_data_32: 0,\n    # No filter by Code.\n    code: 0,\n    # No filter by Timestamp.\n    timestamp_min: 0,\n    # No filter by Timestamp.\n    timestamp_max: 0,\n    # Limit to ten balances at most.\n    limit: 10,\n    flags: %AccountFilterFlags{\n      # Include transfer from the debit side.\n      debits: true,\n      # Include transfer from the credit side.\n      credits: true,\n      # Sort by timestamp in reverse-chronological order.\n      reversed: true\n    }\n  }\n\n{:ok, account_balances} = Connection.get_account_balances(:tb, filter)\n```","ref":"walkthrough.html#get-account-balances"},{"type":"extras","title":"Query Accounts - Walkthrough","doc":"NOTE: This is a preview API that is subject to breaking changes once TigerBeetle has a stable querying API.\n\nQuery accounts by the intersection of some fields and by timestamp range.\n\nThe accounts in the response are sorted by `timestamp` in chronological or reverse-chronological order.\n\n```elixir\nalias TigerBeetlex.QueryFilter\nalias TigerBeetlex.QueryFilterFlags\n\nfilter =\n  %QueryFilter{\n    # Filter by UserData.\n    user_data_128: <<1000::128>>,\n    user_data_64: 100,\n    user_data_32: 10,\n    # Filter by Code.\n    code: 1,\n    # No filter by Ledger.\n    ledger: 0,\n    # No filter by Timestamp.\n    timestamp_min: 0,\n    # No filter by Timestamp.\n    timestamp_max: 0,\n    # Limit to ten transfers at most.\n    limit: 10,\n    # Sort by timestamp in reverse-chronological order.\n    flags: %QueryFilterFlags{reversed: true}\n  }\n\n{:ok, query_accounts} = Connection.query_accounts(:tb, filter)\n```","ref":"walkthrough.html#query-accounts"},{"type":"extras","title":"Query Transfers - Walkthrough","doc":"NOTE: This is a preview API that is subject to breaking changes once TigerBeetle has a stable querying API.\n\nQuery transfers by the intersection of some fields and by timestamp range.\n\nThe transfers in the response are sorted by `timestamp` in chronological or reverse-chronological order.\n\n```elixir\nfilter =\n  %QueryFilter{\n    # Filter by UserData.\n    user_data_128: <<1000::128>>,\n    user_data_64: 100,\n    user_data_32: 10,\n    # Filter by Code.\n    code: 1,\n    # No filter by Ledger.\n    ledger: 0,\n    # No filter by Timestamp.\n    timestamp_min: 0,\n    # No filter by Timestamp.\n    timestamp_max: 0,\n    # Limit to ten transfers at most.\n    limit: 10,\n    # Sort by timestamp in reverse-chronological order.\n    flags: %QueryFilterFlags{reversed: true}\n  }\n\n{:ok, query_transfers} = Connection.query_transfers(:tb, filter)\n```","ref":"walkthrough.html#query-transfers"},{"type":"extras","title":"Linked Events - Walkthrough","doc":"When the `linked` flag is specified for an account when creating accounts or a transfer when creating transfers, it links that event with the next event in the batch, to create a chain of events, of arbitrary length, which all succeed or fail together. The tail of a chain is denoted by the first event without this flag. The last event in a batch may therefore never have the `linked` flag set as this would leave a chain open-ended. Multiple chains or individual events may coexist within a batch to succeed or fail independently.\n\nEvents within a chain are executed within order, or are rolled back on error, so that the effect of each event in the chain is visible to the next, and so that the chain is either visible or invisible as a unit to subsequent events after the chain. The event that was the first to break the chain will have a unique error result. Other events in the chain will have their error result set to `linked_event_failed`.\n\n```elixir\nbatch = [\n  # An individual transfer (successful):\n  %Transfer{id: ID.from_int(1)},\n\n  # A chain of 4 transfers (the last transfer in the chain closes the chain with linked=false):\n  # Commit/rollback. \n  %Transfer{id: ID.from_int(2), flags: %TransferFlags{linked: true}},\n  # Commit/rollback.\n  %Transfer{id: ID.from_int(3), flags: %TransferFlags{linked: true}},\n  # Fail with exists.\n  %Transfer{id: ID.from_int(2), flags: %TransferFlags{linked: true}},\n  # Fail without committing.\n  %Transfer{id: ID.from_int(4)},\n\n  # An individual transfer (successful)\n  # This should not see any effect from the failed chain above\n  %Transfer{id: ID.from_int(2)},\n\n  # A chain of 2 transfers (the first transfer fails the chain):\n  %Transfer{id: ID.from_int(2), flags: %TransferFlags{linked: true}},\n  %Transfer{id: ID.from_int(3)},\n\n  # A chain of 2 transfers (successful):\n  %Transfer{id: ID.from_int(3), flags: %TransferFlags{linked: true}},\n  %Transfer{id: ID.from_int(4)}\n]\n\n{:ok, _transfer_errors} = Connection.create_transfers(:tb, batch)\n# Error handling omitted.\n```","ref":"walkthrough.html#linked-events"},{"type":"extras","title":"Imported Events - Walkthrough","doc":"When the `imported` flag is specified for an account when creating accounts or a transfer when creating transfers, it allows importing historical events with a user-defined timestamp.\n\nThe entire batch of events must be set with the flag `imported`.\n\nIt’s recommended to submit the whole batch as a `linked` chain of events, ensuring that if any event fails, none of them are committed, preserving the last timestamp unchanged. This approach gives the application a chance to correct failed imported events, re-submitting the batch again with the same user-defined timestamps.\n\n```elixir\n# External source of time.\nhistorical_timestamp = 0\n# Events loaded from an external source.\n# Loaded from an external source.\nhistorical_accounts = []\n# Loaded from an external source.\nhistorical_transfers = []\n\n# First, load and import all accounts with their timestamps from the historical source.\n{reversed_accounts, historical_timestamp} =\n  Enum.reduce(historical_accounts, {[], historical_timestamp}, fn _historical_account,\n                                                                  {acc, last_timestamp} ->\n    # Set a unique and strictly increasing timestamp.\n    timestamp = last_timestamp + 1\n\n    account = %Account{\n      timestamp: timestamp,\n      # To ensure atomicity, the entire batch (except the last event in the chain)\n      # must be `linked`.\n      flags: %AccountFlags{imported: true, linked: true}\n      # Bring over other fields from historical_account\n    }\n\n    {[account | acc], timestamp}\n  end)\n\naccounts =\n  reversed_accounts\n  # We \"unset\" the linked flag to the \"last\" event in the chain, which is actually the\n  # first one since we're accumulating by prepending\n  |> List.update_at(0, fn account ->\n    %{account | flags: %AccountFlags{imported: true}}\n  end)\n  # We reverse everything to obtain the correct order\n  |> Enum.reverse()\n\n{:ok, _account_errors} = Connection.create_accounts(:tb, accounts)\n# Error handling omitted.\n\n# Then, load and import all transfers with their timestamps from the historical source.\n{reversed_transfers, _historical_timestamp} =\n  Enum.reduce(historical_transfers, {[], historical_timestamp}, fn _historical_account,\n                                                                   {acc, last_timestamp} ->\n    # Set a unique and strictly increasing timestamp.\n    timestamp = last_timestamp + 1\n\n    transfer = %Transfer{\n      timestamp: timestamp,\n      # To ensure atomicity, the entire batch (except the last event in the chain)\n      # must be `linked`.\n      flags: %AccountFlags{imported: true, linked: true}\n      # Bring over other fields from historical_account\n    }\n\n    {[transfer | acc], timestamp}\n  end)\n\ntransfers =\n  reversed_transfers\n  # We \"unset\" the linked flag to the \"last\" event in the chain, which is actually the\n  # first one since we're accumulating by prepending\n  |> List.update_at(0, fn account ->\n    %{account | flags: %TransferFlags{imported: true}}\n  end)\n  # We reverse everything to obtain the correct order\n  |> Enum.reverse()\n\n{:ok, _transfer_errors} = Connection.create_transfers(:tb, transfers)\n# Error handling omitted.\n```\n\nSince it is a `linked` chain, in case of any error the entire batch is rolled back and can be retried with the same historical timestamps without regressing the cluster timestamp.","ref":"walkthrough.html#imported-events"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}